using System;
using System.Text;
using System.IO;
using System.Net;
using System.Collections.Generic;
using Terrasoft.Nui.ServiceModel.DataContract;
using Terrasoft.Core.Entities;
using System.Web.Script.Serialization;
using Terrasoft.Common;

namespace DataServiceSelectExample
{
    class Program
    {
        // Основной URL приложения bpm'online. Необходимо заменить на пользовательский.
        private const string baseUri = @"http://example.bpmonline.com";
       // Строка запроса к методу Login сервиса AuthService.svc.
        private const string authServiceUri = baseUri + @"/ServiceModel/AuthService.svc/Login";
        // Строка пути запроса SelectQuery.
        private const string selectQueryUri = baseUri + @"/0/DataService/json/SyncReply/SelectQuery";
        // Cookie аутентификации bpm'online.
        private static CookieContainer AuthCookie = new CookieContainer();


        // Метод выполняет аутентификацию пользователя.
        // Параметры:
        // userName — имя пользователя bpm'online,
        // userPassword — пароль пользователя bpm'online.
        private static bool TryLogin(string userName, string userPassword)
        {
            // Создание экземпляра запроса к сервису аутентификации.
            var authRequest = HttpWebRequest.Create(authServiceUri) as HttpWebRequest;
            // Определение метода HTTP-запроса.
            authRequest.Method = "POST";
            // Определение типа контента запроса.
            authRequest.ContentType = "application/json";
            // Включение использования cookie в запросе.
            authRequest.CookieContainer = AuthCookie;
            // Помещение в тело запроса учетной информации пользователя.
            using (var requesrStream = authRequest.GetRequestStream())
            {
                using (var writer = new StreamWriter(requesrStream))
                {
                    writer.Write(@"{
                        ""UserName"":""" + userName + @""",
                        ""UserPassword"":""" + userPassword + @"""
                    }");
                }
            }
            // Получение ответа от сервера. Если аутентификация проходит успешно, в свойство AuthCookie будут 
            // помещены cookie, которые могут быть использованы для последующих запросов.
            using (var response = (HttpWebResponse)authRequest.GetResponse())
            {
                if (AuthCookie.Count > 0)
                {
                    return true;
                }
            }
            return false;
        }

        // Главный метод приложения.
        static void Main(string[] args)
        {
            // Если выполнение аутентификации неудачно, приложение завершает работу.
            if (!TryLogin("Пользователь 1", "Пользователь 1"))
            {
                Console.WriteLine("Ошибка аутентификации!");
                return;
            }

            // Экземпляр класса запроса.
            var selectQuery = new SelectQuery()
            {
                // Название корневой схемы.
                RootSchemaName = "Contact",
                // Коллекция колонок запроса.
                Columns = new SelectQueryColumns()
            };
            // Выражение, задающее тип колонки [[ФИО].
            var columnExpressionName = new ColumnExpression()
            {
                // Тип выражения — колонка схемы.
                ExpressionType = EntitySchemaQueryExpressionType.SchemaColumn,
                // Путь к колонке.
                ColumnPath = "Name"
            };
            // Конфигурирование колонки [Name]. 
            var selectQueryColumnName = new SelectQueryColumn()
            {
                //Загловок.
                Caption = "ФИО",
                // Направление сортировки — по возрастанию.
                OrderDirection = OrderDirection.Ascending,
                // Позиция порядка сортировки.
                OrderPosition = 0,
                // Выражение, задающее тип колонки.
                Expression = columnExpressionName
            };
            // Выражение, задающее тип колонки [Количество активностей].
            var columnExpressionActivitiesCount = new ColumnExpression()
            {
                // Тип выражения — вложенный запрос.
                ExpressionType = EntitySchemaQueryExpressionType.SubQuery,
                // Путь к колонке относительно корневой схемы.
                ColumnPath = "[Activity:Contact].Id",
                // Тип функции — агрегирующая.
                FunctionType = FunctionType.Aggregation,
                // Тип агрегации — количество.
                AggregationType = AggregationType.Count
            };
            // Конфигурирование колонки [Количество активностей].
            var selectQueryColumnActivitiesCount = new SelectQueryColumn()
            {
                //Загловок.
                Caption = "Количество активностей",
                // Направление сортировки — по возрастанию.
                OrderDirection = OrderDirection.Ascending,
                // Позиция порядка сортировки.
                OrderPosition = 1,
                // Выражение, задающее тип колонки.
                Expression = columnExpressionActivitiesCount
            };

            // Добавление колонок в запрос.
            selectQuery.Columns.Items = new Dictionary<string, SelectQueryColumn>()
            {
                {
                    "Name",
                    selectQueryColumnName
                },
                {
                    "ActivitiesCount",
                    selectQueryColumnActivitiesCount
                }
            };
            // Сериализация экземпляра класса запроса на добавление в JSON-строку.
            var json = new JavaScriptSerializer().Serialize(selectQuery);
            // Преобразование строки JSON-объекта в массив байтов.
            byte[] jsonArray = Encoding.UTF8.GetBytes(json);
            // Создание экземпляра HTTP-запроса.
            var selectRequest = HttpWebRequest.Create(selectQueryUri) as HttpWebRequest;
            // Определение метода запроса.
            selectRequest.Method = "POST";
            // Определение типа содержимого запроса.
            selectRequest.ContentType = "application/json";
            // Добавление полученных ранее аутентификационных cookie в запрос на получение данных.
            selectRequest.CookieContainer = AuthCookie;
            // Установить длину содержимого запроса.
            selectRequest.ContentLength = jsonArray.Length;
            // Помещение JSON-объекта в содержимое запроса .
            using (var requestStream = selectRequest.GetRequestStream())
            {
                requestStream.Write(jsonArray, 0, jsonArray.Length);
            }
            // Выполнение HTTP-запроса и получение ответа от сервера.
            using (var response = (HttpWebResponse)selectRequest.GetResponse())
            {
                // Вывод ответа в консоль.
                using (StreamReader reader = new StreamReader(response.GetResponseStream()))
                {
                    Console.WriteLine(reader.ReadToEnd());
                }
            }
            // Задержка выполнения приложения.
            Console.ReadKey();
         }
     }
 }
